(function (global, factory) {
    if (typeof define === "function" && define.amd) {
        define(['exports', 'react', 'prop-types', 'moment', 'melon-core/classname/cxBuilder', '../util', './Header', './Clock'], factory);
    } else if (typeof exports !== "undefined") {
        factory(exports, require('react'), require('prop-types'), require('moment'), require('melon-core/classname/cxBuilder'), require('../util'), require('./Header'), require('./Clock'));
    } else {
        var mod = {
            exports: {}
        };
        factory(mod.exports, global.react, global.propTypes, global.moment, global.cxBuilder, global.util, global.Header, global.Clock);
        global.Panel = mod.exports;
    }
})(this, function (exports, _react, _propTypes, _moment, _cxBuilder, _util, _Header, _Clock) {
    'use strict';

    Object.defineProperty(exports, "__esModule", {
        value: true
    });

    var _react2 = _interopRequireDefault(_react);

    var _propTypes2 = _interopRequireDefault(_propTypes);

    var _moment2 = _interopRequireDefault(_moment);

    var Util = _interopRequireWildcard(_util);

    var _Header2 = _interopRequireDefault(_Header);

    var _Clock2 = _interopRequireDefault(_Clock);

    function _interopRequireWildcard(obj) {
        if (obj && obj.__esModule) {
            return obj;
        } else {
            var newObj = {};

            if (obj != null) {
                for (var key in obj) {
                    if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key];
                }
            }

            newObj.default = obj;
            return newObj;
        }
    }

    function _interopRequireDefault(obj) {
        return obj && obj.__esModule ? obj : {
            default: obj
        };
    }

    function _classCallCheck(instance, Constructor) {
        if (!(instance instanceof Constructor)) {
            throw new TypeError("Cannot call a class as a function");
        }
    }

    var _createClass = function () {
        function defineProperties(target, props) {
            for (var i = 0; i < props.length; i++) {
                var descriptor = props[i];
                descriptor.enumerable = descriptor.enumerable || false;
                descriptor.configurable = true;
                if ("value" in descriptor) descriptor.writable = true;
                Object.defineProperty(target, descriptor.key, descriptor);
            }
        }

        return function (Constructor, protoProps, staticProps) {
            if (protoProps) defineProperties(Constructor.prototype, protoProps);
            if (staticProps) defineProperties(Constructor, staticProps);
            return Constructor;
        };
    }();

    function _possibleConstructorReturn(self, call) {
        if (!self) {
            throw new ReferenceError("this hasn't been initialised - super() hasn't been called");
        }

        return call && (typeof call === "object" || typeof call === "function") ? call : self;
    }

    function _inherits(subClass, superClass) {
        if (typeof superClass !== "function" && superClass !== null) {
            throw new TypeError("Super expression must either be null or a function, not " + typeof superClass);
        }

        subClass.prototype = Object.create(superClass && superClass.prototype, {
            constructor: {
                value: subClass,
                enumerable: false,
                writable: true,
                configurable: true
            }
        });
        if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass;
    }

    var cx = (0, _cxBuilder.create)('TimePickerPanel');

    /**
     * melon/TimePickerPanel
     *
     * @extends {React.Component}
     * @class
     */

    var TimePickerPanel = function (_Component) {
        _inherits(TimePickerPanel, _Component);

        /**
         * 构造函数
         *
         * @constructor
         * @public
         * @param  {*} props 组件属性
         */
        function TimePickerPanel(props) {
            _classCallCheck(this, TimePickerPanel);

            var _this = _possibleConstructorReturn(this, (TimePickerPanel.__proto__ || Object.getPrototypeOf(TimePickerPanel)).call(this, props));

            _this.onModeChange = _this.onModeChange.bind(_this);
            _this.onTimeChange = _this.onTimeChange.bind(_this);

            /**
             * 初始状态
             *
             * @private
             * @type {Object}
             */
            _this.state = {
                time: props.time,
                mode: props.mode
            };
            return _this;
        }

        /**
         * react 组件生命周期——属性更新
         *
         * @public
         * @override
         * @param {Object} nextProps 更新的属性
         */


        _createClass(TimePickerPanel, [{
            key: 'componentWillReceiveProps',
            value: function componentWillReceiveProps(nextProps) {

                var time = nextProps.time;

                if (!Util.isSame(time, this.props.time)) {
                    this.setState({ time: time });
                }
            }
        }, {
            key: 'shouldComponentUpdate',
            value: function shouldComponentUpdate(nextProps, nextState) {
                return !Util.isSame(nextState.time, this.state.time) || nextState.mode !== this.state.mode;
            }
        }, {
            key: 'onModeChange',
            value: function onModeChange(_ref) {
                var mode = _ref.mode;

                this.setState({ mode: mode });
            }
        }, {
            key: 'onTimeChange',
            value: function onTimeChange(_ref2) {
                var _this2 = this;

                var time = _ref2.time,
                    mode = _ref2.mode;
                var _props = this.props,
                    begin = _props.begin,
                    end = _props.end;


                time = Util.closest(time, begin, end);

                var nextState = {
                    time: time,
                    mode: mode || this.state.mode
                };

                this.setState(nextState, function () {
                    _this2.props.onChange({
                        time: time
                    });
                });
            }
        }, {
            key: 'render',
            value: function render() {
                var _state = this.state,
                    time = _state.time,
                    mode = _state.mode;
                var _props2 = this.props,
                    begin = _props2.begin,
                    end = _props2.end;


                return _react2.default.createElement(
                    'div',
                    { className: cx(this.props).build() },
                    _react2.default.createElement(_Header2.default, {
                        time: time,
                        mode: mode,
                        onModeChange: this.onModeChange,
                        onChange: this.onTimeChange }),
                    _react2.default.createElement(_Clock2.default, {
                        time: time,
                        mode: mode,
                        begin: begin,
                        end: end,
                        onChange: this.onTimeChange })
                );
            }
        }]);

        return TimePickerPanel;
    }(_react.Component);

    exports.default = TimePickerPanel;


    TimePickerPanel.displayName = 'TimePickerPanel';

    TimePickerPanel.defaultProps = {
        time: (0, _moment2.default)().millisecond(0).seconds(0).toDate(),
        mode: 'hour'
    };

    TimePickerPanel.propTypes = {
        time: _propTypes2.default.instanceOf(Date),
        mode: _propTypes2.default.oneOf(['hour', 'minute']),
        begin: _propTypes2.default.instanceOf(Date),
        end: _propTypes2.default.instanceOf(Date),
        format: _propTypes2.default.string,
        onChange: _propTypes2.default.func.isRequired
    };
});
//# sourceMappingURL=Panel.js.map
